﻿
Imports System.Drawing
Imports System.Windows.Forms

Public Class ByteEdit

    '----------------------------------------------------------
    '
    '       ポート値
    '
    '----------------------------------------------------------
    Private m_bitValue As Byte
    Public Property Value() As Byte
        Get
            Return m_bitValue
        End Get
        Set(ByVal value As Byte)
            m_bitValue = value
            m_label(2).Text = value
            m_label(3).Text = String.Format("(&&H{0:X2})", value)
            Invalidate()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       バイト位置
    '
    '----------------------------------------------------------
    Private m_valuePosition As Integer
    Public Property ValuePosition() As Integer
        Get
            Return m_valuePosition
        End Get
        Set(ByVal value As Integer)
            m_valuePosition = value
            m_label(0).Text = value
            m_label(1).Text = String.Format("(&&H{0:X4})", value)
        End Set
    End Property


    '----------------------------------------------------------
    '
    '       ラベル表示イネーブル
    '
    '----------------------------------------------------------
    Private m_labelEnable As Boolean = True
    Public Property LabelEnable() As Boolean
        Get
            Return m_labelEnable
        End Get
        Set(ByVal value As Boolean)
            m_labelEnable = value
            m_label(0).Visible = value
            m_label(1).Visible = value
            LayoutComponent()
        End Set
    End Property


    '----------------------------------------------------------
    '
    '       ビットイメージのサイズ
    '
    '----------------------------------------------------------
    Private m_imageSize As Size = New Size(32, 32)
    Property ImageSize() As Size
        Get
            Return m_imageSize
        End Get
        Set(ByVal value As Size)
            m_imageSize = value
            If m_imageSize.Width = 0 Or m_imageSize.Height = 0 Then m_imageSize = New Size(32, 32)
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットイメージ間の距離(ピクセル）
    '
    '----------------------------------------------------------
    Private m_imageGap As Integer = 10
    Public Property ImageGap() As Integer
        Get
            Return m_imageGap
        End Get
        Set(ByVal value As Integer)
            m_imageGap = value
            LayoutComponent()
        End Set
    End Property

    '----------------------------------------------------------
    '
    '       ビットイメージの表示形
    '
    '----------------------------------------------------------
    Public Enum ImageFormType
        CIRCLE
        RECTANGLE
        SQUARE
    End Enum
    Private m_imageForm As ImageFormType = ImageFormType.CIRCLE
    Public Property ImageForm() As Integer
        Get
            Return m_imageForm
        End Get
        Set(ByVal value As Integer)
            m_imageForm = value
            LayoutComponent()
        End Set
    End Property

    '
    '
    '
    Private m_label() As Label = {New Label, New Label, New Label, New Label}

    Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        ' InitializeComponent() 呼び出しの後で初期化を追加します。
        m_label(0).Size = New Size(48, 16)
        m_label(0).TextAlign = ContentAlignment.MiddleRight
        m_label(1).Size = New Size(56, 16)
        m_label(1).TextAlign = ContentAlignment.MiddleLeft
        m_label(2).Size = New Size(32, 16)
        m_label(2).TextAlign = ContentAlignment.MiddleRight
        m_label(3).Size = New Size(40, 16)
        m_label(3).TextAlign = ContentAlignment.MiddleLeft
        Controls.Add(m_label(0))
        Controls.Add(m_label(1))
        Controls.Add(m_label(2))
        Controls.Add(m_label(3))
    End Sub

    '----------------------------------------------------------
    '
    '       フォーム読込み時の初期設定
    '
    '----------------------------------------------------------
    Private Sub PortView_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        LayoutComponent()                   '部品内のレイアウトを整える
    End Sub

    '----------------------------------------------------------
    '
    '       部品内レイアウトの計算
    '
    '----------------------------------------------------------
    Protected Sub LayoutComponent()
        Height = ImageSize.Height + 4
        m_label(0).Height = ImageSize.Height
        m_label(1).Height = ImageSize.Height
        m_label(2).Height = ImageSize.Height
        m_label(3).Height = ImageSize.Height
        m_label(0).Left = 2
        m_label(1).Left = m_label(0).Left + m_label(0).Width + 4
        m_label(2).Left = m_label(1).Left + m_label(1).Width + ImageGap * 10 + ImageSize.Width * 8 + 6
        m_label(3).Left = m_label(2).Left + m_label(2).Width + 4
        m_label(0).Top = (Height - m_label(0).Height) / 2
        m_label(1).Top = (Height - m_label(1).Height) / 2
        m_label(2).Top = (Height - m_label(2).Height) / 2
        m_label(3).Top = (Height - m_label(3).Height) / 2
        Width = m_label(3).Left + m_label(3).Width + 4
    End Sub

    '----------------------------------------------------------
    '
    '       ペイント・イベント処理
    '
    '----------------------------------------------------------
    Private Sub ByteEdit_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        Dim circleLeft As Integer
        Dim fillBrush As System.Drawing.Brush
        Dim linePen As Pen
        For index As Integer = 0 To 7
            circleLeft = m_label(1).Left + m_label(1).Width + (ImageGap + ImageSize.Width) * (8 - index) - ImageSize.Width / 2
            If (Value And (1 << index)) <> 0 Then
                fillBrush = Brushes.Red
                linePen = New Pen(Color.Red, 2)
            Else
                fillBrush = Brushes.Gray
                linePen = New Pen(Color.Black, 2)
            End If
            If ImageForm = ImageFormType.CIRCLE Then
                e.Graphics.FillEllipse(fillBrush, circleLeft, 2, ImageSize.Width, ImageSize.Height)
            ElseIf ImageForm = ImageFormType.RECTANGLE Then
                e.Graphics.FillRectangle(fillBrush, circleLeft, 2, ImageSize.Width, ImageSize.Height)
            Else
                If (Value And (1 << index)) <> 0 Then
                    e.Graphics.DrawLine(Pens.Gray, circleLeft - ImageGap \ 2, ImageSize.Height, circleLeft - ImageGap \ 2, 5)
                    'e.Graphics.DrawLine(Pens.Gray, circleLeft - ImageGap \ 2, ImageSize.Height \ 2 + 3, circleLeft - ImageGap \ 2, 5)
                    e.Graphics.DrawLine(Pens.Gray, circleLeft + ImageSize.Width + ImageGap \ 2, 4, circleLeft + ImageSize.Width + ImageGap \ 2, ImageSize.Height)
                    'e.Graphics.DrawLine(Pens.Gray, circleLeft + ImageSize.Width + ImageGap \ 2, 4, circleLeft + ImageSize.Width + ImageGap \ 2, ImageSize.Height \ 2 + 3)
                    e.Graphics.DrawLine(linePen, circleLeft - ImageGap \ 2, 5, circleLeft + ImageSize.Width + ImageGap \ 2, 5)
                Else
                    e.Graphics.DrawLine(Pens.Gray, circleLeft - ImageGap \ 2, ImageSize.Height \ 2 + 3, circleLeft - ImageGap \ 2, ImageSize.Height)
                    e.Graphics.DrawLine(Pens.Gray, circleLeft + ImageSize.Width + ImageGap \ 2, ImageSize.Height, circleLeft + ImageSize.Width + ImageGap \ 2, ImageSize.Height \ 2 + 3)
                    e.Graphics.DrawLine(linePen, circleLeft - ImageGap \ 2, ImageSize.Height, circleLeft + ImageSize.Width + ImageGap \ 2, ImageSize.Height)
                End If
            End If
        Next
    End Sub


    '----------------------------------------------------------
    '
    '       ビット値の反転
    '
    '----------------------------------------------------------
    Public Sub NotBit(ByVal bitPos As Integer)
        Value = Value Xor (1 << bitPos)
    End Sub



    '----------------------------------------------------------
    '
    '       ポート値変更イベントの定義
    '
    '----------------------------------------------------------
    Public Event PortValueChanged(ByVal sender As ByteEdit, ByVal bitPosition As Integer)

    '----------------------------------------------------------
    '
    '       ポート値変更時、ビット位置を求めてイベント生成
    '
    '----------------------------------------------------------
    Private Sub ByteEdit_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseClick
        Dim bitPos As Integer = 1
        bitPos = 7 - (e.X - (m_label(1).Left + m_label(1).Width) - (ImageSize.Width + ImageGap) \ 2) \ (ImageSize.Width + ImageGap)
        If (bitPos <= 7) And (bitPos >= 0) Then
            RaiseEvent PortValueChanged(Me, bitPos)
        End If
    End Sub
End Class
